//---------------------------------------------------------------------------
// Copyright (C) 1998 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//---------------------------------------------------------------------------
//
//  MLanPkt.H - Constants and globals for MicroLan repeater packet 
//              interpretation and processing.
//
//  Version: 1.00
//

// single-byte commands operations, enumerate from 0x80
enum { CMD_ML_RESET = 0x80, CMD_ML_SEARCH, CMD_ML_ACCESS,
       CMD_ML_OVERDRIVE_ACCESS, CMD_RESET, CMD_GETBUF, CMD_ERROR, 
       MaxSingleCommand
     };

// mult-byte commands, enumerate from 0x00
enum { DATA_ID = 0x00, DATA_SEARCH_STATE, DATA_SEARCH_CMD, 
       DATA_MODE, DATA_CAPABILITY, DATA_OUTBOUND_MAX, 
       DATA_INBOUND_MAX, DATA_PROTOCOL, DATA_VENDOR,
       CMD_ML_BIT, CMD_ML_DATA, CMD_DELAY, MaxMultiCommand
     };

// return codes, enumerate from 0x00
enum { RET_SUCCESS = 0x00, RET_END_SEARCH, RET_BUSY, 
       RET_ERROR, RET_NO_DEVICE, RET_ML_SHORTED,
       RET_OUTBOUND_OVERRUN, RET_INBOUND_OVERRUN,
       RET_REG_OVERRUN, RET_END_OF_INBOUND, RET_READ_ONLY, 
       RET_WRITE_ONLY, RET_CMD_UNKNOWN, MaxRetCode 
     };

// size of data registers
#define SIZE_DATA_ID                8
#define SIZE_DATA_SEARCH_STATE      2
#define SIZE_DATA_SEARCH_CMD        1
#define SIZE_DATA_MODE              1 
#define SIZE_DATA_CAPABILITY        1 
#define SIZE_DATA_OUTBOUND_MAX      1
#define SIZE_DATA_INBOUND_MAX       1
#define SIZE_DATA_PROTOCOL          20 
#define SIZE_DATA_VENDOR            20

// size of frame registers
#define MAX_INFRAME     48      
#define MAX_OUTFRAME    48      

// mode bit flags
#define MODE_NORMAL     0x00
#define MODE_OVERDRIVE  0x01
#define MODE_STRONG5    0x02
#define MODE_PROGRAM    0x04
#define MODE_BREAK      0x08

// delay times
#define DELAY_32        0x00
#define DELAY_64        0x01
#define DELAY_128       0x02
#define DELAY_256       0x03
#define DELAY_512       0x04
#define DELAY_1024      0x05
#define DELAY_2048      0x06
#define DELAY_4096      0x07
#define DELAY_MS        0x80
#define DELAY_US        0x00

// search commands
#define SEARCH_NORMAL   0xF0
#define SEARCH_ALARM    0xEC

// misc contants
#define TRUE            1
#define FALSE           0
#define SINGLE_BYTE_CMD 0x80
#define INBOUND         0
#define OUTBOUND        1

// typedefs
typedef unsigned char uchar;

// Packet Globals
#ifdef MLANPKT  

   // data registers (13 bytes)
   uchar id[SIZE_DATA_ID];
   uchar search_state[SIZE_DATA_SEARCH_STATE];
   uchar search_cmd[SIZE_DATA_SEARCH_CMD];
   uchar mode[SIZE_DATA_MODE];
   // could be constant
   uchar capability[SIZE_DATA_CAPABILITY];
   uchar outbound_max[SIZE_DATA_OUTBOUND_MAX];
   uchar inbound_max[SIZE_DATA_INBOUND_MAX];
   uchar protocol[SIZE_DATA_PROTOCOL];
   uchar vendor[SIZE_DATA_VENDOR];
   // global state
   uchar last_cmd;
   uchar last_cmd_return;
   uchar LastDeviceFlag;

   // inbound and outbound packet buffers
   uchar inbound[MAX_INFRAME+1], outbound[MAX_OUTFRAME+1];

#else
   
   // data registers (13 bytes)
   extern uchar id[SIZE_DATA_ID];
   extern uchar search_state[SIZE_DATA_SEARCH_STATE];
   extern uchar search_cmd[SIZE_DATA_SEARCH_CMD];
   extern uchar mode[SIZE_DATA_MODE];
   // could be constant
   extern uchar capability[SIZE_DATA_CAPABILITY];
   extern uchar outbound_max[SIZE_DATA_OUTBOUND_MAX];
   extern uchar inbound_max[SIZE_DATA_INBOUND_MAX];
   extern uchar protocol[SIZE_DATA_PROTOCOL];
   extern uchar vendor[SIZE_DATA_VENDOR];
   // global state
   extern uchar last_cmd;
   extern uchar last_cmd_return;
   extern uchar LastDeviceFlag;

   // inbound and outbound packet buffers and lengths  
   extern uchar inbound[], outbound[];
 
#endif  // MLANPKT

