//---------------------------------------------------------------------------
// Copyright (C) 1998 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//---------------------------------------------------------------------------
//
//  TstMLPkt.C - Test application to test MicroLan packet processing
//               using the low level TMEX 32-bit functions in TMEXLnkp.c.
//
//  Version: 1.00
//

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include "mlanpkt.h"

// global TMEX variables
int SessionHandle;

// external TMEX functions need for testing 
extern long  far pascal TMExtendedStartSession(short, short, void *);
extern short far pascal TMValidSession(long); 
extern short far pascal TMEndSession(long); 
extern short far pascal TMSetup(long);
extern short far pascal TMClose(long);

// external Host packet functions
extern int MLanHostPacketReceive(uchar *);
extern void MLanHostPacketSend(uchar *);
extern uchar MLanRepeaterReset(void);

/*----------------------------------------------------------------------
 *  Main Test
 */
void main(int argc, char **argv)
{
   short PortNum=1,PortType=2;
   uchar SendPkt[256];
   uchar RecPkt[256];
   int recLen,i,testcnt=0;
   uchar LastFamilyDiscrepancy;
   uchar ID_EE[8], ID_TMP[8], sendLen;

   //------------------ TMEX start functions
   SessionHandle = TMExtendedStartSession(PortNum,PortType,NULL);
   
   // check the session handle
   if (SessionHandle <= 0)
   {
      printf("Could not get TMEX session handle, aborting.\n");
      exit(1);
   }

   // setup the port
   if (TMSetup(SessionHandle) != 1)
   {
      printf("Error setting up MicroLan port, aborting.\n");
      exit(1);
   }
   //------------------ End of TMEX start functions

   //----------------------------------------
   // Introduction
   printf("\n/---------------------------------------------\n");
   printf("  The following is a test excersize of the\n"
          "  MicroLan Repeater Version 1.00.\n\n"
          "  This test was run with iButton-TMEX version\n"
          "  3.10 using a DS1410E adapter with 2 DS1920's (DS1820),\n"
          "  1 DS1971 (DS2430), and 1 DS1996.\n\n"
          "  This test uses the minimum sizes of data \n"
          "  registers and frame buffers as described\n"
          "  in the specification.\n\n");

   // reset the repeater
   MLanRepeaterReset();

   //----------------------------------------
   // Test Read MicroLan Repeater capabilities and version
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: Read MicroLan Repeater capabilities and version\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // read data registers, version, capability, buffer max, out max
   SendPkt[sendLen++] = DATA_CAPABILITY;
   SendPkt[sendLen++] = 0;
   SendPkt[sendLen++] = DATA_OUTBOUND_MAX;
   SendPkt[sendLen++] = 0;
   SendPkt[sendLen++] = DATA_INBOUND_MAX;
   SendPkt[sendLen++] = 0;
   SendPkt[sendLen++] = DATA_PROTOCOL;
   SendPkt[sendLen++] = 0;
   SendPkt[sendLen++] = DATA_VENDOR;
   SendPkt[sendLen++] = 0;
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);

   //----------------------------------------
   // Test Find and read the ID of the first MicroLan device
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: Find and read the ID of the first MicroLan device\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // clear the search state so we find the 'first' device
   SendPkt[sendLen++] = DATA_SEARCH_STATE;
   SendPkt[sendLen++] = 2;
   SendPkt[sendLen++] = 0;
   SendPkt[sendLen++] = 0;
   // do a reset,search and then read results
   SendPkt[sendLen++] = CMD_ML_RESET;
   SendPkt[sendLen++] = CMD_ML_SEARCH;
   SendPkt[sendLen++] = DATA_ID;
   SendPkt[sendLen++] = 0;
   SendPkt[sendLen++] = DATA_SEARCH_STATE;
   SendPkt[sendLen++] = 0;
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);
   // print result
   // get the LastFamilyDiscrepancy
   LastFamilyDiscrepancy = RecPkt[recLen - 1]; 
   printf("result: ID of device found ");
   // get the ID of the DS1920 (DS1820) found 
   for (i = 0; i < 8; i++)
   {
      ID_TMP[i] = RecPkt[recLen - 12 + i];
      printf("%02X ",ID_TMP[i]);
   }
   printf("\n");

   //----------------------------------------
   // Test Skip the family type found (0x10) and find the next
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: Skip the family type found (0x10) and find the next\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // set search state to skip the current family type
   SendPkt[sendLen++] = DATA_SEARCH_STATE;
   SendPkt[sendLen++] = 2;
   SendPkt[sendLen++] = LastFamilyDiscrepancy;
   SendPkt[sendLen++] = 0;
   // do a reset,search and then read results
   SendPkt[sendLen++] = CMD_ML_RESET;
   SendPkt[sendLen++] = CMD_ML_SEARCH;
   SendPkt[sendLen++] = DATA_ID;
   SendPkt[sendLen++] = 0;
   SendPkt[sendLen++] = DATA_SEARCH_STATE;
   SendPkt[sendLen++] = 0;
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);
   // print result
   printf("result: ID of device found ");
   // get the ID of the DS2430 found 
   for (i = 0; i < 8; i++)
   {
      ID_EE[i] = RecPkt[recLen - 12 + i];
      printf("%02X ",ID_EE[i]);
   }
   printf("\n");

   //----------------------------------------
   // Test write 5 bytes to the DS2430's scratchpad and read back
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: write 5 bytes to the DS2430's scratchpad \n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // access the current device with address in DATA_ID
   SendPkt[sendLen++] = CMD_ML_ACCESS;
   // construct a block of communication to MicroLan
   SendPkt[sendLen++] = CMD_ML_DATA;
   SendPkt[sendLen++] = 8; // block length
   SendPkt[sendLen++] = 7; // data lenght
   // send the write scratchpad command
   SendPkt[sendLen++] = 0x0F;
   // send the adress byte
   SendPkt[sendLen++] = 0;
   // the 5 bytes of data to write
   for (i = 0; i < 5; i++)
      SendPkt[sendLen++] = (uchar)(i + 0x5A);
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);
   // print result
   printf("result: bytes written 5A 5B 5C 5D 5E\n");

   //----------------------------------------
   // Test read 5 bytes of the DS2430's scratchpad to verify correct
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: read 5 bytes of the DS2430's scratchpad to verify correct\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // access the current device with address in DATA_ID
   SendPkt[sendLen++] = CMD_ML_ACCESS;
   // construct a block of communication to MicroLan
   SendPkt[sendLen++] = CMD_ML_DATA;
   SendPkt[sendLen++] = 3; // block length
   SendPkt[sendLen++] = 7; // data lenght slave read 5 0xFF's
   // send the read scratchpad command
   SendPkt[sendLen++] = 0xAA;
   // send the adress byte
   SendPkt[sendLen++] = 0;
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);
   // print result
   printf("result: bytes read back from scratchpad ");
   for (i = 0; i < 5; i++)
      printf("%02X ",RecPkt[recLen - 5 + i]);
   printf("\n");

   //----------------------------------------
   // Test copy scratchpad of DS2430 to memory with strong pull-up
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: copy scratchpad of DS2430 to memory with strong pull-up\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // access the current device with address in DATA_ID
   SendPkt[sendLen++] = CMD_ML_ACCESS;
   // construct a block of communication to MicroLan
   SendPkt[sendLen++] = CMD_ML_DATA;
   SendPkt[sendLen++] = 3; // block length
   SendPkt[sendLen++] = 2; // data lenght 
   // send the copy scratchpad command
   SendPkt[sendLen++] = 0x55;
   // send the validation key
   SendPkt[sendLen++] = 0xA5;
   // turn on strong pull-up power delivery 
   SendPkt[sendLen++] = DATA_MODE;
   SendPkt[sendLen++] = 1;
   SendPkt[sendLen++] = MODE_STRONG5;
   // delay for 128ms
   SendPkt[sendLen++] = CMD_DELAY;
   SendPkt[sendLen++] = 1;
   SendPkt[sendLen++] = DELAY_128 | DELAY_MS;
   // turn off strong pull-up power delivery 
   SendPkt[sendLen++] = DATA_MODE;
   SendPkt[sendLen++] = 1;
   SendPkt[sendLen++] = MODE_NORMAL;
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);

   //----------------------------------------
   // Test read the 32 bytes memory of the DS2430
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: read the 32 bytes of memory of the DS2430 found\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // access the current device with address in DATA_ID
   SendPkt[sendLen++] = CMD_ML_ACCESS;
   // construct a block of communication to MicroLan
   SendPkt[sendLen++] = CMD_ML_DATA;
   SendPkt[sendLen++] = 3;  // block length
   SendPkt[sendLen++] = 34; // data lenght with 32 slave reads 
   // send the read memory command
   SendPkt[sendLen++] = 0xF0;
   // send the adress byte
   SendPkt[sendLen++] = 0;
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);
   // print result
   printf("result: DS2430 memory read 32 bytes ");
   for (i = 0; i < 32; i++)
      printf("%02X ",RecPkt[recLen - 32 + i]);
   printf("\n");

   //----------------------------------------
   // Test read the first byte of memory on the DS2430 using bits
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: read the first byte of memory on the DS2430 using bits\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // access the current device with address in DATA_ID
   SendPkt[sendLen++] = CMD_ML_ACCESS;
   // construct a block of communication to MicroLan
   SendPkt[sendLen++] = CMD_ML_DATA;
   SendPkt[sendLen++] = 3; // block length
   SendPkt[sendLen++] = 2; // data lenght 
   // send the read memory command
   SendPkt[sendLen++] = 0xF0;
   // send the adress byte
   SendPkt[sendLen++] = 0;
   // send 8 read bits
   SendPkt[sendLen++] = CMD_ML_BIT;
   SendPkt[sendLen++] = 8;
   for (i = 0; i < 8; i++)
      SendPkt[sendLen++] = 0x01;
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);
   // print result
   printf("result: first byte in DS2430 memory (binary) ");
   for (i = 0; i < 8; i++)
      printf("%1X",RecPkt[recLen - 8 + i]);
   printf("\n");

   //----------------------------------------
   // Test make sure the DS1920 is on the MicroLan 
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: make sure the DS1920 is on the MicroLan\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // set the DATA_ID to the tempature device
   SendPkt[sendLen++] = DATA_ID;
   SendPkt[sendLen++] = 8;
   for (i = 0; i < 8; i++)
      SendPkt[sendLen++] = ID_TMP[i];
   // set the search state so we finds the same device
   SendPkt[sendLen++] = DATA_SEARCH_STATE;
   SendPkt[sendLen++] = 2;
   SendPkt[sendLen++] = 64;
   SendPkt[sendLen++] = 0;
   // do a reset,search and then read results
   SendPkt[sendLen++] = CMD_ML_RESET;
   SendPkt[sendLen++] = CMD_ML_SEARCH;
   SendPkt[sendLen++] = DATA_ID;
   SendPkt[sendLen++] = 0;
   SendPkt[sendLen++] = DATA_SEARCH_STATE;
   SendPkt[sendLen++] = 0;
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);
   // print result
   for (i = 0; i < 8; i++)
      if (ID_TMP[i] != RecPkt[recLen - 12 + i])
         break;
   printf("result: DS1920 present on MicroLan: ");
   if (i == 8) 
      printf(" TRUE\n");
   else 
      printf(" FALSE\n");

   //----------------------------------------
   // Test instruct the DS1920 to read the temperature
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: instruct the DS1920 to read the temperature\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // access the current device with address in DATA_ID
   SendPkt[sendLen++] = CMD_ML_ACCESS;
   // send block of communication to MicroLan
   SendPkt[sendLen++] = CMD_ML_DATA;
   SendPkt[sendLen++] = 2; // block length
   SendPkt[sendLen++] = 1; // data lenght 
   // send the convert temperature command
   SendPkt[sendLen++] = 0x44;
   // turn on strong pull-up power delivery 
   SendPkt[sendLen++] = DATA_MODE;
   SendPkt[sendLen++] = 1;
   SendPkt[sendLen++] = MODE_STRONG5;
   // delay for 1024ms
   SendPkt[sendLen++] = CMD_DELAY;
   SendPkt[sendLen++] = 1;
   SendPkt[sendLen++] = DELAY_1024 | DELAY_MS;
   // turn off strong pull-up power delivery 
   SendPkt[sendLen++] = DATA_MODE;
   SendPkt[sendLen++] = 1;
   SendPkt[sendLen++] = MODE_NORMAL;
   // access the current device with address in DATA_ID
   SendPkt[sendLen++] = CMD_ML_ACCESS;
   // construct a block of communication to MicroLan
   SendPkt[sendLen++] = CMD_ML_DATA;
   SendPkt[sendLen++] = 2; // block length
   SendPkt[sendLen++] = 3; // data lenght with 2 slave reads 
   // send the read scratchpad command
   SendPkt[sendLen++] = 0xBE;
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);
   // print result
   printf("result: DS1920 temperature read: %d C\n", (RecPkt[recLen - 2] |
           (RecPkt[recLen - 1] << 8)) / 2);

   //----------------------------------------
   // Test target the family type (0x0C) and search for an ID
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: target the family type (0x0C) and search for an ID\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // set search state to target a family type
   SendPkt[sendLen++] = DATA_SEARCH_STATE;
   SendPkt[sendLen++] = 2;
   SendPkt[sendLen++] = 9;
   SendPkt[sendLen++] = 0;
   // set the DATA_ID to family code 0x0C
   SendPkt[sendLen++] = DATA_ID;
   SendPkt[sendLen++] = 1;
   SendPkt[sendLen++] = 0x0C;
   // do a reset,search and then read results
   SendPkt[sendLen++] = CMD_ML_RESET;
   SendPkt[sendLen++] = CMD_ML_SEARCH;
   SendPkt[sendLen++] = DATA_ID;
   SendPkt[sendLen++] = 0;
   SendPkt[sendLen++] = DATA_SEARCH_STATE;
   SendPkt[sendLen++] = 0;
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);
   // print result
   printf("result: ID of device found ");
   // get the ID of the DS1996 found 
   for (i = 0; i < 8; i++)
   {
      ID_EE[i] = RecPkt[recLen - 12 + i];
      printf("%02X ",ID_EE[i]);
   }
   printf("\n");

   //----------------------------------------
   // Test put DS1996 in overdrive and read the first 32 bytes
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: put DS1996 in overdrive and read the first 32 bytes\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // read the mode
   SendPkt[sendLen++] = DATA_MODE;
   SendPkt[sendLen++] = 0;
   // access the current device with address in DATA_ID and put in ovedrive
   SendPkt[sendLen++] = CMD_ML_OVERDRIVE_ACCESS;
   // read the mode
   SendPkt[sendLen++] = DATA_MODE;
   SendPkt[sendLen++] = 0;
   // read the first 32 bytes of the DS1996 in overdrive
   // construct a block of communication to MicroLan
   SendPkt[sendLen++] = CMD_ML_DATA;
   SendPkt[sendLen++] = 4;  // block length
   SendPkt[sendLen++] = 35; // data lenght with 32 slave reads
   // send read memory command
   SendPkt[sendLen++] = 0xF0;
   // send address TA1 and TA2
   SendPkt[sendLen++] = 0x00;
   SendPkt[sendLen++] = 0x00;
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);
   // print result
   printf("result: DS1996 memory read next 32 bytes ");
   for (i = 0; i < 32; i++)
      printf("%02X ",RecPkt[recLen - 32 + i]);
   printf("\n");

   //----------------------------------------
   // Test continue reading DS1996 memory 32 bytes
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: continue reading DS1996 memory 32 bytes\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // set the block length 32 bytes of slave 0xFF's
   SendPkt[sendLen++] = CMD_ML_DATA;
   SendPkt[sendLen++] = 1;  // block length
   SendPkt[sendLen++] = 32; // data lenght with 32 slave reads
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);
   // print result
   printf("result: DS1996 memory read next 32 bytes ");
   for (i = 0; i < 32; i++)
      printf("%02X ",RecPkt[recLen - 32 + i]);
   printf("\n");

   //----------------------------------------
   // Test continue reading DS1996 memory 32 bytes polling for result
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: continue reading DS1996 memory 32 bytes polling for result\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // set the block length 32 bytes of slave 0xFF's
   SendPkt[sendLen++] = CMD_ML_DATA;
   SendPkt[sendLen++] = 1;  // block length
   SendPkt[sendLen++] = 32; // data lenght with 32 slave reads
   // turn back to normal mode
   SendPkt[sendLen++] = DATA_MODE;
   SendPkt[sendLen++] = 1;
   SendPkt[sendLen++] = MODE_NORMAL;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   printf("\n**Send another packet to request the result.\n");
   sendLen = 1; // reserve first byte for length
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);

   //----------------------------------------
   // Test parsing error, outbound too big, no previous command
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: Parsing error, outbound too big, no previous command\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // read 47 bytes 
   SendPkt[sendLen++] = CMD_ML_DATA;
   SendPkt[sendLen++] = 1;  // block length
   SendPkt[sendLen++] = 47; // data lenght with 47 slave reads
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);

   //----------------------------------------
   // Test parsing error, outbound too big, previous single command
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: Parsing error, outbound too big, previous single command\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // reset the MicroLan
   SendPkt[sendLen++] = CMD_ML_RESET;
   // read 47 bytes 
   SendPkt[sendLen++] = CMD_ML_DATA;
   SendPkt[sendLen++] = 1;  // block length
   SendPkt[sendLen++] = 47; // data lenght with 47 slave reads
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);

   //----------------------------------------
   // Test parsing error, ran out of inbound data
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: Parsing error, ran out of inbound data\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // set DATA_ID with less data then length
   SendPkt[sendLen++] = DATA_ID;
   SendPkt[sendLen++] = 9;
   SendPkt[sendLen++] = 0x01;
   SendPkt[sendLen++] = 0x02;
   SendPkt[sendLen++] = 0x03;
   SendPkt[sendLen++] = 0x04;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   printf("\n**Send another packet to request the result.\n");
   sendLen = 1; // reserve first byte for length
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);

   //----------------------------------------
   // Test parsing error, write length too big for data register
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: Parsing error, write length too big for data register\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // set DATA_ID with 9 bytes of data (max is 8)
   SendPkt[sendLen++] = DATA_ID;
   SendPkt[sendLen++] = 9;
   for (i = 0; i < 9; i++)
      SendPkt[sendLen++] = 0x00;
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);

   //----------------------------------------
   // Test parsing error, invalid multi-byte command
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: Parsing error, invalid multi-byte command\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // read non-existant multi-byte command
   SendPkt[sendLen++] = 0x23;
   SendPkt[sendLen++] = 0;
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);
   
   //----------------------------------------
   // Test parsing error, invalid single-byte command
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: Parsing error, invalid single-byte command\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // read non-existant single-byte command
   SendPkt[sendLen++] = 0x97;
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);

   //----------------------------------------
   // Test parsing error, writing read-only data register
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: Parsing error, writing read-only data register\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   SendPkt[sendLen++] = DATA_PROTOCOL;
   SendPkt[sendLen++] = 1;
   SendPkt[sendLen++] = 0x55;
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);

   //----------------------------------------
   // Test parsing error, reading write-only command
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: Parsing error, reading write-only command\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   SendPkt[sendLen++] = CMD_ML_DATA;
   SendPkt[sendLen++] = 0;
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);

   //------------------ TMEX end functions
   TMClose(SessionHandle);
   TMEndSession(SessionHandle);
   //------------------ End of TMEX end functions
}


