//---------------------------------------------------------------------------
// Copyright (C) 1998 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//---------------------------------------------------------------------------
//
//  TstMLPktE.C - Test application to test MicroLan packet processing
//                using the Win32 COM port functions in Win32Lnk.C. Test
//                application to program a byte in an EPROM iButton.  
//
//  Version: 1.00
//

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include "mlanpkt.h"

// defines 
#define FALSE 0
#define TRUE  1

// global TMEX variables
int SessionHandle;

// external COM functions for Win32 test
extern int  OpenWin32COM(int COMPrt);
extern void CloseWin32COM(void);
extern int DS2480Detect(void);

// external Host packet functions
extern int MLanHostPacketReceive(uchar *);
extern void MLanHostPacketSend(uchar *);
extern uchar MLanRepeaterReset(void);

//----------------------------------------------------------------------
//  Main Test
//
void main(int argc, char **argv)
{
   short PortNum=1;
   uchar SendPkt[256];
   uchar RecPkt[256];
   int recLen,i,testcnt=0;
   uchar LastFamilyDiscrepancy;
   uchar sendLen;

   //------------------ Win32 COM start functions
   if (OpenWin32COM(PortNum))
      printf("COM%d opened using Win32\n",PortNum);
   else
   {
      printf("Could not open COM%d using Win32, aborting.\n",PortNum);
      exit(1);
   }

   // detect DS2480
   if (DS2480Detect())
      printf("DS2480 detected\n");
   else
   {
      printf("DS2480 not detected, aborting\n");
      CloseWin32COM();
      exit(1);
   }      
   //------------------ End of Win32 Start functions

   //----------------------------------------
   // Introduction
   printf("\n/---------------------------------------------\n");
   printf("  The following is a test excersize of the\n"
          "  MicroLan Repeater Version 1.00.\n\n"
          "  This test was run on Windows 95 COM port\n"
          "  using a DS9097U-25 adapter (+ 12v) with \n"
          "  1 DS1982 (DS2502).\n\n"
          "  This test uses the minimum sizes of data \n"
          "  registers and frame buffers as described\n"
          "  in the specification.\n\n");

   // reset the repeater
   MLanRepeaterReset();

   //----------------------------------------
   // Test Read MicroLan Repeater capabilities and version
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: Read MicroLan Repeater capabilities and version\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // read data registers, version, capability, buffer max, out max
   SendPkt[sendLen++] = DATA_CAPABILITY;
   SendPkt[sendLen++] = 0;
   SendPkt[sendLen++] = DATA_OUTBOUND_MAX;
   SendPkt[sendLen++] = 0;
   SendPkt[sendLen++] = DATA_INBOUND_MAX;
   SendPkt[sendLen++] = 0;
   SendPkt[sendLen++] = DATA_PROTOCOL;
   SendPkt[sendLen++] = 0;
   SendPkt[sendLen++] = DATA_VENDOR;
   SendPkt[sendLen++] = 0;
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);

   //----------------------------------------
   // Test Find and read the ID of the first MicroLan device
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: Find and read the ID of the first MicroLan device\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // clear the search state so we find the 'first' device
   SendPkt[sendLen++] = DATA_SEARCH_STATE;
   SendPkt[sendLen++] = 2;
   SendPkt[sendLen++] = 0;
   SendPkt[sendLen++] = 0;
   // do a reset,search and then read results
   SendPkt[sendLen++] = CMD_ML_RESET;
   SendPkt[sendLen++] = CMD_ML_SEARCH;
   SendPkt[sendLen++] = DATA_ID;
   SendPkt[sendLen++] = 0;
   SendPkt[sendLen++] = DATA_SEARCH_STATE;
   SendPkt[sendLen++] = 0;
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);
   // print result
   // get the LastFamilyDiscrepancy
   LastFamilyDiscrepancy = RecPkt[recLen - 1]; 
   printf("result: ID of device found ");
   // get the ID of the device found 
   for (i = 0; i < 8; i++)
      printf("%02X ",RecPkt[recLen - 12 + i]);
   printf("\n");

   //----------------------------------------
   // Test program 1 byte of the DS1982 (DS2502) found 
   printf("\n/---------------------------------------------\n");
   printf("TEST %d: program 1 byte of the DS1982 (DS2502) found\n",testcnt++);
   sendLen = 1; // reserve first byte for length
   // access the current device with address in DATA_ID
   SendPkt[sendLen++] = CMD_ML_ACCESS;
   // send block of communication to MicroLan
   SendPkt[sendLen++] = CMD_ML_DATA;
   SendPkt[sendLen++] = 5;    // block length
   SendPkt[sendLen++] = 5;    // data length (4 write, 1 read)
   SendPkt[sendLen++] = 0x0F; // write memory command 
   SendPkt[sendLen++] = 0;    // address 1 
   SendPkt[sendLen++] = 0;    // address 2 
   SendPkt[sendLen++] = 0x55; // data to program
   // turn on program voltage 
   SendPkt[sendLen++] = DATA_MODE;
   SendPkt[sendLen++] = 1;
   SendPkt[sendLen++] = MODE_PROGRAM;
   // delay for 512us
   SendPkt[sendLen++] = CMD_DELAY;
   SendPkt[sendLen++] = 1;
   SendPkt[sendLen++] = DELAY_512 | DELAY_US;
   // turn off program voltage 
   SendPkt[sendLen++] = DATA_MODE;
   SendPkt[sendLen++] = 1;
   SendPkt[sendLen++] = MODE_NORMAL;
   // send block of communication to MicroLan
   SendPkt[sendLen++] = CMD_ML_DATA;
   SendPkt[sendLen++] = 1;    // block length
   SendPkt[sendLen++] = 1;    // 1 read of data result
   // request the result buffer as the last command
   SendPkt[sendLen++] = CMD_GETBUF;
   // set the length
   SendPkt[0] = sendLen - 1;
   // send and receive the frame
   MLanHostPacketSend(SendPkt);
   recLen = MLanHostPacketReceive(RecPkt);
   // print result
   printf("result: byte result of program: %02X\n", RecPkt[recLen - 1]);

   //------------------ Win32 COM end functions
   CloseWin32COM();
   //------------------ End of Win32 end functions
}


